#include <iostream.h>
	
	//Cdigo fuente de las pginas 198-202
	// Programa que implementa un TAD cadena

	/****************************************************/
	/*                    Declaracin                   */
	/****************************************************/

	class	cadena {
	private:
		static const char	Error;
		static char	C_Error;	// Copia del error
		char	*s;
		int	Tam;
	public:
		// No hay constructor por defecto

		// Destructor:
		~cadena();

		// Constructores de conversin:

		cadena(int , char= ' ');
		// cadena(N, c): Construye una cadena de N elementos
		// rellenados con c

		cadena(const char * );
		// Constructor de conversin implcita de const char * 
		// a cadena.

		cadena(char );
		// Construye una cadena con un solo carcter

		// Constructor copia:
		cadena(const cadena & );

		// Operadores de asignacin:
		const cadena &operator = (const cadena & );
		const cadena &operator = (const char * ); // Y conversin

		// Operador de conversin:
		operator const char * () const; // Convierte a strings

		// Mtodos constantes:
		int	Long() const;
		const cadena operator + (const cadena & ) const;
		// Concatena dos cadenas

		// Miembros no constantes:
		const cadena & operator += (const cadena & );
		// Concatena una cadena al final

		char & operator [] (int );
		// Operador de acceso a vector (en este caso una cadena)

		static void	(*F_Error)(const char * );
		// Funcin que se llama cuando hay un error
	};


	/****************************************************/
	/*                  Implementacin                  */
	/****************************************************/
	
	#include	<cstring>		// strlen(), strcpy()

	const char cadena::Error= -1;
	char cadena::C_Error;

	inline cadena::~cadena() {
		delete s;
	}

	cadena::cadena(int i, char c) {
		Tam= i;
		s= new char[Tam + 1];	// Reserva
		while(i--)
			s[i]= c;
		s[Tam]= '\0';
	}

	cadena::cadena(const char * p) {
		Tam= strlen(p);
		s= new char[Tam + 1];
		(void)strcpy(s, p);
		// Copia todos los caracteres. Incluido el '\0'
	}

	cadena::cadena(char c) {
		Tam= 1;
		s= new char[2];
		s[0]= c;
		s[1]= '\0';
	}

	cadena::cadena(const cadena &c) {
		Tam= c.Tam;
		s= new char[Tam + 1];
		(void)strcpy(s, c.s);	
		// Copia todos los caracteres. Incluido el '\0'
	}

	const cadena & cadena::operator = (const cadena & c) {
		if (this != & c) {
			// Si no soy yo. En este caso no hara falta 
			// porque se compara el tamao

			if (Tam != c.Tam) {	
				// Se tiene que reajustar la longitud
				delete s;
				Tam= c.Tam;
				s= new char[Tam + 1];
			}
			(void)strcpy(s, c.s);	
			// Copia todos los caracteres. Incluido el '\0'
		}
		return *this;
	}

	const cadena & cadena::operator = (const char * p) {
		int	t= strlen(p);
		if (Tam != t) {	
			// Se tiene que reajustar la longitud
			delete s;
			Tam= t;
			s= new char[Tam + 1];
		}
		(void)strcpy(s, p);	
		// Copia todos los caracteres. Incluido el '\0'
		return *this;
	}

	inline cadena::operator const char * () const {
		return s;
	}

	inline int cadena::Long() const {
		return Tam;
	}

	const cadena cadena::operator + (const cadena & c) const {
  		cadena Temp(Tam + c.Tam);

  		char *Aux= Temp.s;
  		(void)strcpy(Aux, s);		
		// Copia todos los caracteres. Incluido el '\0'
  		Aux+= Tam;               // Avanzar hasta el final
  		(void)strcpy(Aux, c);    // Aadir la otra cadena

  		return	Temp;
	}

	const cadena &cadena::operator += (const cadena & c) {
		// Es ms rpido redefinir todo que utilizar operator +
		*this= *this + c;
		return *this; // Podemos retornar *this por referencia
	}

	char & cadena::operator [] (int i) {
		if (i < Tam)
			return s[i];
		else {
			(*F_Error)("ACCESO FUERA DE RANGO");
			return C_Error= Error;	
			// no podemos retornar Error porque es constante 
			// y se podra modificar
		}
	}


	/****************************************************/
	/*                      Prueba                      */
	/****************************************************/

	#include	<iostream>

	void	Aviso(const char * s) {
		cout << "\nMe ha llegado un error de cadena que dice: " 
		     << s << '\n';
	}

	void	(*cadena::F_Error)(const char *)= Aviso;

	void main() {
  		cadena s1(20, '['), s2(20, ']');
  		cadena c1(" Bravo "), c2("a "), c3("todos ");

  		cadena s= s1 + c1;
  		s+= c2;
  		s+= c3;
  		s+= "los ";
  		s+= "lectores! ";
  		s+= s2;
  		s+= '\n';

  		cout << endl << endl << endl;
  		cout << s;

  		cadena t("Por haber hecho el esfuerzo de teclear este "
		         "programa. Repito:\n");
  		cadena u= t + s;
  		cout << u;

  		int	i= 87;
  		cout << "El elemento " << i << " de la cadena es: " 
			<< u[i] << endl;
  		u[94]= 'p';
  		u[54]= 'P';
  		cout << "Y despus de modificar algunos caracteres, la "
 		        "cadena queda:\n" << u;
  		cout << u[2000];	// Fuera de rango
	}
