//Cdigo de las pginas 478-480

class Instruccion {
public:                       	
	Instruccion(Interprete *i) {
		pInterprete = i;
	}

	// Guarda el formato de la intruccion
	void PonFormato (TFormatoInst f) {
		formato = f;
	}

    // Funcin virtual pura para ejecutar la instruccin
	virtual void Ejecutar() = 0;

protected:
	TFormatoInst formato;     // Formato de la instruccin
	Interprete *pInterprete;  // Guarda un puntero al pInterprete
};


//La implementacin de las clases asociadas a las instrucciones es la siguiente:
class Mueve:public Instruccion {
public:
	Mueve(Interprete *i) : Instruccion(i) {}
	virtual void Ejecutar();
};

void Mueve::Ejecutar() {
	Operando *op1, *op2;

    // Obtiene los operandos
	op1 = pInterprete->ObtenerOperandos()->Obtener(formato.op1);
	op2 = pInterprete->ObtenerOperandos()->Obtener(formato.op2);

	// Escribe en el op1 el valor leido de op2
	op1->Escribir(op2->Leer());
}

class Suma:public Instruccion {

public:
	Suma(Interprete *i) : Instruccion(i) {}
	virtual void Ejecutar();
};

void Suma::Ejecutar() {
	Operando *op1,*op2;

	// Obtiene operandos
	op1 = pInterprete->ObtenerOperandos()->Obtener(formato.op1);
	op2 = pInterprete->ObtenerOperandos()->Obtener(formato.op2);

	// Sumamos a op1, op2
	*op1 += *op2;
}

class Resta:public Instruccion {
public:
	Resta(Interprete *i) : Instruccion(i) {}
	virtual void Ejecutar();
};

void Resta::Ejecutar() {
	Operando *op1, *op2;

	// Obtiene operandos
	op1 = pInterprete->ObtenerOperandos()->Obtener( formato.op1);
	op2 = pInterprete->ObtenerOperandos()->Obtener( formato.op2);


    // Restamos a op1, op2
	*op1 -= *op2;
}

class Compara:public Instruccion {
public:
	Compara(Interprete *i) : Instruccion(i) {}
	virtual void Ejecutar();
};

void Compara::Ejecutar() {
	Operando *op1, *op2;

    // Obtiene operandos
	op1 = pInterprete->ObtenerOperandos()->Obtener( formato.op1 );
	op2 = pInterprete->ObtenerOperandos()->Obtener( formato.op2 );

    // Compara y cambia el estado del intrprete
	pInterprete->PonerEstado(Comparar(*op1, *op2));
}

class Salta:public Instruccion {
public:
	Salta(Interprete *i) : Instruccion(i) {}
	virtual void Ejecutar();
};

void Salta::Ejecutar() {
	Operando *op1;

	// Obtiene operando
	op1 = pInterprete->ObtenerOperandos()->Obtener(formato.op1);

    // Comprueba que la ltima comparacin es mayor o igual
	if ((pInterprete->LeerEstado() == mayor ) ||
		(pInterprete->LeerEstado() == igual )) {
            // Salta al valor del primer operando
		pInterprete->CambiarPuntero((TPosicion)op1->Leer());
	}
}

class Entrada:public Instruccion {
public:
	Entrada(Interprete *i) : Instruccion(i) {}
	virtual void Ejecutar();
};

void Entrada::Ejecutar() {
	Operando *op1;

	// Obtiene operando
	op1 = pInterprete->ObtenerOperandos()->Obtener( formato.op1);

	// Escribe en el operando el valor ledo desde teclado
	op1 -> Escribir(pInterprete->ObtenerTeclado()->Leer());
}

class Salida:public Instruccion {
public:
	Salida(Interprete *i) : Instruccion(i) {}
	virtual void Ejecutar();
};

void Salida::Ejecutar() {
	Operando *op1;

	// Obtiene operando
	op1 = pInterprete->ObtenerOperandos()->Obtener(formato.op1);

	// Escribe en pantalla el valor del operando
	pInterprete->ObtenerPantalla()->Escribir(op1->Leer());
}

class Fin:public Instruccion {
public:
	Fin(Interprete *i) : Instruccion(i) {}
	virtual void Ejecutar();
};

void Fin::Ejecutar() {
	pInterprete->Fin();
}
